# -*- coding: utf-8 -*-
import ujson
import time
import requests
import zlib

from amplify.agent.context import context
from amplify import __version__
from amplify.agent import Singleton

__author__ = "Mike Belov"
__copyright__ = "Copyright (C) 2015, Nginx Inc. All rights reserved."
__credits__ = ["Mike Belov", "Andrei Belov", "Ivan Poluyanov", "Oleg Mamontov", "Andrew Alexeev"]
__license__ = ""
__maintainer__ = "Mike Belov"
__email__ = "dedm@nginx.com"


class HTTPClient(Singleton):

    def __init__(self):
        config = context.app_config
        self.url = '%s/%s' % (config['cloud']['api_url'], config['credentials']['api_key'])
        self.timeout = float(config['cloud']['api_timeout'])
        self.verify_ssl = config['cloud']['verify_ssl']
        self.gzip = config['cloud']['gzip']

        self.session = requests.Session()
        self.session.headers.update({
            'Content-Type': 'application/json',
            'User-Agent': 'nginx-amplify-agent/%s' % __version__
        })

    def make_request(self, location, method, data=None, timeout=None, json=True):
        url = location if location.startswith('http') else '%s/%s' % (self.url, location)
        timeout = timeout if timeout is not None else self.timeout
        payload = ujson.encode(data) if data else '{}'
        payload = zlib.compress(payload, self.gzip) if self.gzip else payload

        start_time = time.time()
        result, http_code = None, 200
        try:
            if method == 'get':
                r = self.session.get(
                    url,
                    timeout=timeout,
                    verify=self.verify_ssl
                )
            else:
                r = self.session.post(
                    url,
                    data=payload,
                    timeout=timeout,
                    verify=self.verify_ssl
                )
            http_code = r.status_code
            r.raise_for_status()
            result = r.json() if json else r.text
            return result
        except Exception, e:
            context.default_log.error('failed to %s "%s", exception: "%s"' % (method, url, e.message))
            context.default_log.debug('', exc_info=True)
            result = {}
            raise e
        finally:
            end_time = time.time()
            context.default_log.debug(result)
            context.default_log.info(
                "%s %s %s %s %s %.3f" % (method, url, http_code, len(payload), len(result), end_time - start_time)
            )

    def post(self, url, data=None, timeout=None, json=True):
        return self.make_request(url, 'post', data=data, timeout=timeout, json=json)

    def get(self, url, timeout=None, json=True):
        return self.make_request(url, 'get', timeout=timeout, json=json)
